"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var pino = require("pino");
var STACKTRACE_OFFSET = 4;
var LINE_OFFSET = 7;
var FUNC_OFFSET = 0;
var LambdaLogger = /** @class */ (function () {
    function LambdaLogger(svcName, reqId, corrId) {
        this._instance = traceCaller(getBaseLogger(svcName, reqId, corrId));
    }
    LambdaLogger.getInstance = function (svcName, reqId, corrId) {
        var instance = new LambdaLogger(svcName, reqId, corrId);
        return instance._instance;
    };
    LambdaLogger.prototype.getChildInstance = function (svcName, reqId, corrId) {
        var identity = {};
        if (reqId) {
            identity.requestId = reqId;
        }
        if (corrId) {
            identity.correlationId = corrId;
        }
        if (identity) {
            this._instance = this._instance.child(identity);
        }
        return this._instance;
    };
    return LambdaLogger;
}());
exports.default = LambdaLogger;
function traceCaller(pinoInstance) {
    function get(target, name) {
        return name === "asJson" ? asJson : target[name];
    }
    function asJson() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var errStack = Error().stack;
        var fromLine = errStack.split("\n")[STACKTRACE_OFFSET].substr(LINE_OFFSET);
        var from = fromLine.split(" ")[FUNC_OFFSET];
        args[0] = args[0] ?
            Object.assign({ from_src: "" + from.includes("anonymous") ? fromLine : from }, args[0]) :
            { from_src: "" + from.includes("anonymous") ? fromLine : from };
        return pinoInstance.asJson.apply(this, args);
    }
    return new Proxy(pinoInstance, { get: get });
}
function isLambda() {
    return !!((process.env.LAMBDA_TASK_ROOT && process.env.AWS_EXECUTION_ENV) ||
        (process.env.NODE_ENV === "test") ||
        (process.env.NO_PRETTY) ||
        false);
}
function writeLog(chunk) {
    // process.stdout.write(stringifyVersion(chunk));
    process.stdout.write(chunk);
}
function getProductionStream() {
    return {
        write: writeLog
    };
}
function getPrettyPrintedStream() {
    var stream = pino.pretty({ messageKey: "message", forceColor: true });
    stream.pipe(process.stdout);
    return stream;
}
function getBaseLogger(svc, rid, cid) {
    var level = process.env.CONSOLE_LOG_LEVEL || "info";
    var stream = isLambda() ? getProductionStream() : getPrettyPrintedStream();
    var logger = pino({
        level: level,
        messageKey: "message",
        timestamp: pino.stdTimeFunctions.slowTime,
        serializers: {
            error: pino.stdSerializers.err,
            request: pino.stdSerializers.req,
            response: pino.stdSerializers.res,
        },
        base: {
            service: svc,
            lambda: process.env.AWS_LAMBDA_FUNCTION_NAME,
            correlationId: cid,
            requestId: rid
        }
    }, stream);
    return logger;
}
function getLambdaEventSourceType(event) {
    if (event.Records && event.Records[0].cf) {
        return "isCloudfront";
    }
    if (event.configRuleId && event.configRuleName && event.configRuleArn) {
        return "isAwsConfig";
    }
    if (event.Records && (event.Records[0].eventSource === "aws:codecommit")) {
        return "isCodeCommit";
    }
    if (event.authorizationToken === "incoming-client-token") {
        return "isApiGatewayAuthorizer";
    }
    if (event.StackId && event.RequestType && event.ResourceType) {
        return "isCloudFormation";
    }
    if (event.Records && (event.Records[0].eventSource === "aws:ses")) {
        return "isSes";
    }
    if (event.pathParameters && event.pathParameters.proxy) {
        return "isApiGatewayAwsProxy";
    }
    if (event.source === "aws.events") {
        return "isScheduledEvent";
    }
    if (event.awslogs && event.awslogs.data) {
        return "isCloudWatchLogs";
    }
    if (event.Records && (event.Records[0].EventSource === "aws:sns")) {
        return "isSns";
    }
    if (event.Records && (event.Records[0].eventSource === "aws:dynamodb")) {
        return "isDynamoDb";
    }
    if (event.records && event.records[0].approximateArrivalTimestamp) {
        return "isKinesisFirehose";
    }
    if (event.records && event.deliveryStreamArn
        && event.deliveryStreamArn.startsWith("arn:aws:kinesis:")) {
        return "isKinesisFirehose";
    }
    if (event.eventType === "SyncTrigger" && event.identityId && event.identityPoolId) {
        return "isCognitoSyncTrigger";
    }
    if (event.Records && event.Records[0].eventSource === "aws:kinesis") {
        return "isKinesis";
    }
    if (event.Records && event.Records[0].eventSource === "aws:s3") {
        return "isS3";
    }
    if (event.operation && event.message) {
        return "isMobileBackend";
    }
    return "unknown";
}
